<?php

namespace Drupal\xray_audit\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginManager;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;

/**
 * Returns responses for XrayAudit Task routes.
 */
class XrayAuditTaskController extends ControllerBase {

  /**
   * Plugin manager.
   *
   * @var \Drupal\xray_audit\Plugin\XrayAuditTaskPluginManager
   */
  protected $pluginManager;

  /**
   * Constructs of Controller.
   *
   * @param \Drupal\xray_audit\Plugin\XrayAuditTaskPluginManager $plugin_manager
   *   Plugin manager XrayAudit.
   */
  public function __construct(XrayAuditTaskPluginManager $plugin_manager) {
    $this->pluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin_manager.xray_audit_task')
    );
  }

  /**
   * Return a render array with the results of the task.
   *
   * @param string $group_plugin
   *   Group plugin.
   * @param string $task_plugin
   *   Task plugin.
   * @param string|null $operation
   *   Operation.
   *
   * @return array
   *   Render array.
   */
  public function build(string $group_plugin, string $task_plugin, string $operation = NULL): array {
    $task_plugin = $this->getInstancePlugin($task_plugin);
    if ($task_plugin === NULL) {
      throw new NotFoundHttpException();
    }
    $data = $task_plugin->getDataOperationResult($operation);
    return $task_plugin->buildDataRenderArray($data, $operation);

  }

  /**
   * Init a batch process.
   *
   * @param string $task_plugin
   *   Task plugin.
   * @param string $batch_process
   *   Batch proces.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response if the batch is progressive. No return value otherwise.
   */
  public function buildBatchProcess(string $task_plugin, string $batch_process): RedirectResponse {
    $task_plugin = $this->getInstancePlugin($task_plugin);
    if ($task_plugin === NULL) {
      throw new NotFoundHttpException();
    }

    $bacth_method = $task_plugin->getBatchClass($batch_process);
    if ($bacth_method === NULL || !method_exists($task_plugin, $bacth_method)) {
      throw new NotFoundHttpException();
    }

    return $task_plugin->$bacth_method();

  }

  /**
   * Return a render array with the results of the task.
   *
   * @param string $group_plugin
   *   Group plugin.
   * @param string $task_plugin
   *   Task plugin.
   * @param string|null $operation
   *   Operation.
   *
   * @return string
   *   Title.
   */
  public function getTitle(string $group_plugin, string $task_plugin, string $operation = NULL): string {
    $title = '';
    $task_plugin = $this->getInstancePlugin($task_plugin);
    if ($task_plugin === NULL) {
      return $title;
    }
    $title .= $task_plugin->label();
    if ($operation) {
      $title = $task_plugin->getOperations()[$operation]['label'] ?? $task_plugin->label();
    }
    return $title;
  }

  /**
   * Instance the object plugin.
   *
   * @param string $task_plugin
   *   Id plugin.
   *
   * @return \Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface|null
   *   Instance or null.
   */
  protected function getInstancePlugin(string $task_plugin): ?XrayAuditTaskPluginInterface {
    try {
      /** @var \Drupal\xray_audit\Plugin\XrayAuditTaskPluginInterface $task_plugin */
      $task_plugin = $this->pluginManager->createInstance($task_plugin);
      if ($task_plugin instanceof XrayAuditTaskPluginInterface) {
        return $task_plugin;
      }
    } catch (\Exception $error){
      $this->getLogger('xray-audit')->error($error->getMessage());
    }

    return NULL;

  }

}
